from flask_sketch import templates
from flask_sketch.utils import (
    Answers,
    GenericHandler,
    write_tpl,
    pjoin,
)
import os


def web_only_handler(answers: Answers):
    if answers.application_type == "web_only":
        os.makedirs(pjoin(answers.application_project_folder, "site"))
        write_tpl(
            answers.args.project_name,
            "site_web_only_init_tpl",
            templates.site,
            pjoin(answers.application_project_folder, "site", "__init__.py"),
        )
        write_tpl(
            answers.args.project_name,
            "site_web_only_views_tpl",
            templates.site,
            pjoin(answers.application_project_folder, "site", "views.py"),
        )
        return True


def api_only_handler(answers: Answers):
    if answers.application_type == "api_only":
        os.makedirs(
            pjoin(answers.application_project_folder, "api", "resources")
        )
        return True


def web_api_handler(answers: Answers):
    if answers.application_type == "web_and_api":
        os.makedirs(
            pjoin(answers.application_project_folder, "api", "resources")
        )
        os.makedirs(pjoin(answers.application_project_folder, "site"))
        return True


class AppTypeHandler(GenericHandler):
    ...


app_type_handler = AppTypeHandler(
    web_only_handler, api_only_handler, web_api_handler,
)



from flask_sketch import templates  # noqa
from flask_sketch.utils import Answers
from flask_sketch.utils import GenericHandler


def restful_handler(answers: Answers):
    if answers.api_framework == "restful":
        return True


def restx_handler(answers: Answers):
    if answers.api_framework == "restx":
        return True


def restless_handler(answers: Answers):
    if answers.api_framework == "restless":
        return True


def none_handler(answers: Answers):
    if answers.api_framework == "none":
        return True


class ApiFrameworkHandler(GenericHandler):
    ...


api_framework_handler = ApiFrameworkHandler(
    restful_handler, restx_handler, restless_handler, none_handler,
)



from flask_sketch.utils import (
    Answers,
    write_tpl,
    pjoin,
    add_requirements,
    add_dev_requirements,
)
from flask_sketch import templates


def handle_caching(answers: Answers):
    add_requirements(answers.project_folder, "flask-caching")

    answers.settings["development"]["CACHE_TYPE"] = "simple"
    answers.settings["testing"]["CACHE_TYPE"] = "simple"
    answers.settings["production"]["CACHE_TYPE"] = "simple"
    answers.settings["default"]["EXTENSIONS"].extend(
        [f"{answers.args.project_name}.ext.caching:init_app"]
    )

    write_tpl(
        answers.args.project_name,
        "ext_caching_tpl",
        templates.ext,
        pjoin(answers.application_project_folder, "ext", "caching.py"),
    )
    write_tpl(
        answers.args.project_name,
        "examples_caching_tpl",
        templates.examples,
        pjoin(
            answers.application_project_folder,
            "examples",
            "caching_examples.py",
        ),
    )
    write_tpl(
        answers.args.project_name,
        "examples_init_caching_tpl",
        templates.examples,
        pjoin(answers.application_project_folder, "examples", "__init__.py",),
    )


def handle_limiter(answers: Answers):
    add_requirements(answers.project_folder, "flask-limiter")

    answers.settings["default"][
        "RATELIMIT_DEFAULT"
    ] = "200 per day;50 per hour"
    answers.settings["default"]["RATELIMIT_ENABLED"] = True
    answers.settings["development"]["RATELIMIT_ENABLED"] = False
    answers.settings["default"]["EXTENSIONS"].extend(
        [f"{answers.args.project_name}.ext.limiter:init_app"]
    )

    write_tpl(
        answers.args.project_name,
        "ext_limiter_tpl",
        templates.ext,
        pjoin(answers.application_project_folder, "ext", "limiter.py"),
    )
    write_tpl(
        answers.args.project_name,
        "examples_limiter_tpl",
        templates.examples,
        pjoin(
            answers.application_project_folder,
            "examples",
            "limiter_examples.py",
        ),
    )
    write_tpl(
        answers.args.project_name,
        "examples_init_limiter_tpl",
        templates.examples,
        pjoin(answers.application_project_folder, "examples", "__init__.py",),
    )


def handle_migrate(answers: Answers):
    add_requirements(answers.project_folder, "flask-migrate")

    answers.settings["default"]["EXTENSIONS"].extend(
        [f"{answers.args.project_name}.ext.migrate:init_app"]
    )

    write_tpl(
        answers.args.project_name,
        "ext_migrate_tpl",
        templates.ext,
        pjoin(answers.application_project_folder, "ext", "migrate.py"),
    )


def handle_admin(answers: Answers):
    add_requirements(answers.project_folder, "flask-admin")

    answers.settings["default"]["EXTENSIONS"].extend(
        [f"{answers.args.project_name}.ext.admin:init_app"]
    )

    answers.settings["default"]["FLASK_ADMIN_TEMPLATE_MODE"] = "bootstrap3"
    answers.settings["development"][
        "FLASK_ADMIN_NAME"
    ] = f"{answers.args.project_name} (Dev)"
    answers.settings["testing"][
        "FLASK_ADMIN_NAME"
    ] = f"{answers.args.project_name} (Testing)"
    answers.settings["production"][
        "FLASK_ADMIN_NAME"
    ] = answers.args.project_name

    # TODO refact this part to not use a lot of if statements
    if answers.auth_framework == "security_web":
        write_tpl(
            answers.args.project_name,
            "ext_admin_security_tpl",
            templates.ext.admin,
            pjoin(
                answers.application_project_folder,
                "ext",
                "admin",
                "__init__.py",
            ),
        )


def handle_debugtoolbar(answers: Answers):
    add_dev_requirements(answers.project_folder, "flask-debugtoolbar")

    answers.settings["development"]["EXTENSIONS"].extend(
        ["flask_debugtoolbar:DebugToolbarExtension", "dynaconf_merge_unique"]
    )
    answers.settings["development"]["DEBUG_TOOLBAR_ENABLED"] = True
    answers.settings["development"]["DEBUG_TB_INTERCEPT_REDIRECTS"] = False
    answers.settings["development"]["DEBUG_TB_PROFILER_ENABLED"] = True
    answers.settings["development"]["DEBUG_TB_TEMPLATE_EDITOR_ENABLED"] = True
    answers.settings["development"]["DEBUG_TB_PANELS"] = []
    answers.settings["development"]["DEBUG_TB_PANELS"].extend(
        [
            "flask_debugtoolbar.panels.versions.VersionDebugPanel",
            "flask_debugtoolbar.panels.sqlalchemy.SQLAlchemyDebugPanel",
            "flask_debugtoolbar.panels.timer.TimerDebugPanel",
            "flask_debugtoolbar.panels.headers.HeaderDebugPanel",
            "flask_debugtoolbar.panels.request_vars.RequestVarsDebugPanel",
            "flask_debugtoolbar.panels.template.TemplateDebugPanel",
            "flask_debugtoolbar.panels.route_list.RouteListDebugPanel",
            "flask_debugtoolbar.panels.logger.LoggingPanel",
            "flask_debugtoolbar.panels.profiler.ProfilerDebugPanel",
            "flask_debugtoolbar.panels.config_vars.ConfigVarsDebugPanel",
        ]
    )


def handle_features(answers: Answers):
    for feature in answers.features:
        globals()[f"handle_{feature}"](answers)



from flask_sketch.templates import (
    commands,
    config,
    ext,
    models,
    site,
    app,
    examples,
)

from flask_sketch.templates.ext import admin

__all__ = [
    "commands",
    "config",
    "ext",
    "models",
    "site",
    "app",
    "examples",
    "admin",
]



import toml
from flask_sketch import templates
from flask_sketch.utils import (
    Answers,
    GenericHandler,
    write_tpl,
    pjoin,
    add_requirements,
    FlaskSketchTomlEncoder,
)


def clean_settings(settings: dict):
    s = settings
    for k in s:
        s[k] = {
            key: val for key, val in s[k].items() if val != "not_overridden"
        }
    return s


def dynaconf_handler(answers: Answers):
    if answers.config_framework == "dynaconf":
        add_requirements(answers.project_folder, "dynaconf")

        settings_toml = clean_settings(answers.settings)
        secrets_toml = clean_settings(answers.secrets)

        with open(
            pjoin(answers.project_folder, "settings.toml"),
            "w",
            encoding="utf-8",
        ) as file:
            toml.dump(settings_toml, file, encoder=FlaskSketchTomlEncoder())

        with open(
            pjoin(answers.project_folder, ".secrets.toml"),
            "w",
            encoding="utf-8",
        ) as file:
            toml.dump(secrets_toml, file, encoder=FlaskSketchTomlEncoder())

        write_tpl(
            answers.args.project_name,
            "config_dynaconf_tpl",
            templates.config,
            pjoin(
                answers.application_project_folder, "config", "__init__.py",
            ),
        )
        write_tpl(
            answers.args.project_name,
            "app_web_only_dynaconf_tpl",
            templates.app,
            pjoin(answers.application_project_folder, "app.py"),
        )

        return True


def environs_handler(answers: Answers):
    if answers.config_framework == "environs":
        return True


def none_handler(answers: Answers):
    if answers.config_framework == "none":
        return True


class ConfigHandler(GenericHandler):
    ...


config_handler = ConfigHandler(
    dynaconf_handler, environs_handler, none_handler,
)



#!/usr/bin/python3
# from pprint import pprint
import argparse

from pyfiglet import Figlet
from PyInquirer import prompt

from flask_sketch.config import cli_style
from flask_sketch.make import create_project
from flask_sketch.questions import get_questions

parser = argparse.ArgumentParser(description="Flask Sketch CLI")
parser.add_argument("project_name", type=str)
parser.add_argument(
    "-e", action="store_true", help="Create examples endpoints"
)
parser.add_argument(
    "-p",
    action="store_true",
    help="Create a pyproject.toml to work with poetry",
)
parser.add_argument(
    "-v",
    action="store_true",
    help="Create a virtualenv and install requirements",
)

f = Figlet(font="slant")


def flask_sketch(args):
    print(f.renderText("Flask Sketch"))

    answers = prompt(get_questions(), style=cli_style)
    if answers:
        # pprint(answers)
        create_project(args, answers)


if __name__ == "__main__":
    args = parser.parse_args()
    args.project_name = args.project_name.lower()
    flask_sketch(args)



config_questions = [
    {
        "type": "list",
        "message": "Select your Configuration Framework",
        "name": "config_framework",
        "choices": [
            {"name": "Dynaconf", "value": "dynaconf"},
            {
                "name": "Environs",
                "value": "environs",
                "disabled": "Not yet supported",
            },
            {
                "name": "None (just regular env vars)",
                "value": "none",
                "disabled": "Not supported yet",
            },
        ],
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
]



import os
import pathlib
from argparse import Namespace

from flask_sketch.handlers import (
    api_framework_handler,
    app_type_handler,
    auth_handler,
    config_handler,
    database_handler,
    handle_features,
)
from flask_sketch.utils import Answers, cleanup, make_commom, pjoin


def create_project(args: Namespace, asws: dict):
    pf = pjoin(str(pathlib.Path().absolute()), args.project_name)
    apf = pjoin(
        str(pathlib.Path().absolute()),
        args.project_name,
        args.project_name.replace("-", "_"),
    )

    answers = Answers(pf, apf, asws, args)

    make_commom(answers)

    answers.settings["default"]["DEBUG"] = False
    answers.settings["development"]["DEBUG"] = True
    answers.secrets["default"]["SECRET_KEY"] = str(os.urandom(32))

    app_type_handler(answers)
    database_handler(answers)
    auth_handler(answers)
    if "api" in answers.application_type:
        api_framework_handler(answers)
    handle_features(answers)
    config_handler(answers)
    cleanup(answers)

    if args.e:
        os.system(f"python -m venv {pf}/.venv")



commom_questions = [
    {
        "type": "list",
        "message": "Select the type of your project",
        "name": "application_type",
        "choices": [
            {"name": "Web Only (no API)", "value": "web_only"},
            {
                "name": "API Only (no frontend)",
                "value": "api_only",
                "disabled": "Not yet supported",
            },
            {
                "name": "Web and API",
                "value": "web_and_api",
                "disabled": "Soon will be supported.",
            },
        ],
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
    {
        "type": "list",
        "message": "Select your database",
        "name": "database",
        "choices": [
            {"name": "SQLite", "value": "sqlite"},
            {"name": "Postgres", "value": "postgres"},
            {"name": "MySQL", "value": "mysql"},
            {
                "name": "MongoDB",
                "value": "mongodb",
                "disabled": "Not yet supported",
            },
            {
                "name": "None (no database)",
                "value": "none",
                "disabled": "Not yet supported",
            },
        ],
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
]



import os

from flask_sketch.utils import (
    Answers,
    GenericHandler,
    write_tpl,
    pjoin,
    add_requirements,
)
from flask_sketch import templates


def login_handler(answers: Answers):
    if answers.auth_framework == "login_web":
        return True


def security_web_handler(answers: Answers):
    if answers.auth_framework == "security_web":
        add_requirements(
            answers.project_folder, "flask-security-too", "bcrypt"
        )

        answers.settings["default"]["SECURITY_REGISTERABLE"] = True
        answers.settings["default"]["SECURITY_POST_LOGIN_VIEW"] = "/"
        answers.settings["default"]["EXTENSIONS"].extend(
            [f"{answers.args.project_name}.ext.auth:init_app"]
        )

        answers.secrets["default"]["SECURITY_PASSWORD_SALT"] = str(
            os.urandom(12)
        )

        write_tpl(
            answers.args.project_name,
            "security_web_only_tpl",
            templates.commands,
            pjoin(
                answers.application_project_folder, "commands", "__init__.py",
            ),
        )

        write_tpl(
            answers.args.project_name,
            "ext_security_web_only_tpl",
            templates.ext,
            pjoin(answers.application_project_folder, "ext", "auth.py"),
        )

        write_tpl(
            answers.args.project_name,
            "models_security_web_only_tpl",
            templates.models,
            pjoin(answers.application_project_folder, "models", "user.py"),
        )

        write_tpl(
            answers.args.project_name,
            "examples_security_auth_tpl",
            templates.examples,
            pjoin(
                answers.application_project_folder,
                "examples",
                "auth_examples.py",
            ),
        )

        write_tpl(
            answers.args.project_name,
            "examples_init_security_tpl",
            templates.examples,
            pjoin(
                answers.application_project_folder, "examples", "__init__.py",
            ),
        )

        return True


def basicauth_web_handler(answers: Answers):
    if answers.auth_framework == "basicauth_web":
        add_requirements(answers.project_folder, "flask-basicAuth")
        return True


def praetorian_handler(answers: Answers):
    if answers.auth_framework == "praetorian":
        add_requirements(answers.project_folder, "flask-praetorian")
        return True


def jwt_extended_handler(answers: Answers):
    if answers.auth_framework == "jwt_extended":
        add_requirements(answers.project_folder, "flask-jwt-extended")
        return True


def basicauth_api_handler(answers: Answers):
    if answers.auth_framework == "basicauth_api":
        add_requirements(answers.project_folder, "flask-basicauth")
        return True


def security_web_api_handler(answers: Answers):
    if answers.auth_framework == "security_web_api":
        add_requirements(
            answers.project_folder, "flask-security-too", "flask-jwt-extended"
        )
        return True


def login_jwt_extended_handler(answers: Answers):
    if answers.auth_framework == "login_jwt_extended":
        add_requirements(
            answers.project_folder, "flask-login", "flask-jwt-extended"
        )
        return True


def basicauth_web_api_handler(answers: Answers):
    if answers.auth_framework == "basicauth_web_api":
        add_requirements(answers.project_folder, "flask-BasicAuth")
        return True


def none_handler(answers: Answers):
    if answers.auth_framework == "none":
        if not answers.database == "none":
            write_tpl(
                answers.args.project_name,
                "no_auth_tpl",
                templates.commands,
                pjoin(
                    answers.application_project_folder,
                    "commands",
                    "__init__.py",
                ),
            )

        return True


class AuthHandler(GenericHandler):
    def __call__(self, answers: Answers):
        for handler in self.handlers:
            r = handler(answers)
            if r:
                if not handler.__name__ == "none_handler":
                    write_tpl(
                        answers.args.project_name,
                        "models_auth_tpl",
                        templates.models,
                        pjoin(
                            answers.application_project_folder,
                            "models",
                            "__init__.py",
                        ),
                    )
                return r


auth_handler = AuthHandler(
    login_handler,
    security_web_handler,
    basicauth_web_handler,
    praetorian_handler,
    jwt_extended_handler,
    basicauth_api_handler,
    security_web_api_handler,
    login_jwt_extended_handler,
    basicauth_web_api_handler,
    none_handler,
)



from PyInquirer import Token, style_from_dict

cli_style = style_from_dict(
    {
        Token.QuestionMark: "#E91E63 bold",
        Token.Selected: "#673AB7 bold",
        Token.Instruction: "",  # default
        Token.Answer: "#2196f3 bold",
        Token.Question: "",
    }
)



import importlib.resources as pkg_resources
import os
from argparse import Namespace
from typing import Callable

import toml

from flask_sketch import templates


class Answers:
    def __init__(self, pf: str, apf: str, answers: dict, args: Namespace):
        self.project_folder = pf
        self.application_project_folder = apf
        self.application_type: str = answers.get("application_type")
        self.database: str = answers.get("database")
        self.auth_framework: str = answers.get("auth_framework")
        self.api_framework: str = answers.get("api_framework")
        self.config_framework: str = answers.get("config_framework")
        self.features: list = answers.get("features")
        self.args = args
        self.secrets = {
            "default": {
                "SECRET_KEY": "not_overridden",
                "SECURITY_PASSWORD_SALT": "not_overridden",
            },
        }
        self.settings = {
            "default": {
                "DEBUG": "not_overridden",
                "SQLALCHEMY_TRACK_MODIFICATIONS": "not_overridden",
                "SECURITY_REGISTERABLE": "not_overridden",
                "SECURITY_POST_LOGIN_VIEW": "not_overridden",
                "FLASK_ADMIN_TEMPLATE_MODE": "not_overridden",
                "RATELIMIT_DEFAULT": "not_overridden",
                "RATELIMIT_ENABLED": "not_overridden",
                "EXTENSIONS": [],
            },
            "development": {
                "DEBUG": "not_overridden",
                "SQLALCHEMY_DATABASE_URI": "not_overridden",
                "SQLALCHEMY_TRACK_MODIFICATIONS": "not_overridden",
                "FLASK_ADMIN_NAME": "not_overridden",
                "CACHE_TYPE": "not_overridden",
                "RATELIMIT_ENABLED": "not_overridden",
                "EXTENSIONS": [],
                "DEBUG_TOOLBAR_ENABLED": "not_overridden",
                "DEBUG_TB_INTERCEPT_REDIRECTS": "not_overridden",
                "DEBUG_TB_PROFILER_ENABLED": "not_overridden",
                "DEBUG_TB_TEMPLATE_EDITOR_ENABLED": "not_overridden",
                "DEBUG_TB_PANELS": "not_overridden",
            },
            "testing": {
                "FLASK_ADMIN_NAME": "not_overridden",
                "SQLALCHEMY_DATABASE_URI": "not_overridden",
                "CACHE_TYPE": "not_overridden",
            },
            "production": {
                "FLASK_ADMIN_NAME": "not_overridden",
                "CACHE_TYPE": "not_overridden",
                "SQLALCHEMY_DATABASE_URI": "not_overridden",
            },
        }


class GenericHandler:
    def __init__(self, *handlers: Callable):
        self.handlers = handlers

    def __call__(self, answers: Answers):
        for handler in self.handlers:
            r = handler(answers)
            if r:
                return r


class FlaskSketchTomlEncoder(toml.TomlEncoder):
    def __init__(self, _dict=dict, preserve=False, separator=",\r\n"):
        super(FlaskSketchTomlEncoder, self).__init__(_dict, preserve)
        if separator.strip() == "":
            separator = "," + separator
        elif separator.strip(" \t\n\r,"):
            raise ValueError("Invalid separator for arrays")
        self.separator = separator

    def dump_list(self, v):
        t = []
        retval = "[\r\n"
        for u in v:
            t.append(self.dump_value(u))
        while t != []:
            s = []
            for u in t:
                if isinstance(u, list):
                    for r in u:
                        s.append(r)
                else:
                    retval += "\t" + str(u) + self.separator
            t = s
        retval += "]"
        return retval


def has_answers(answers: dict, have: dict = {}, not_have: dict = {}):

    for da in have:
        if not answers.get(da).lower() in have.get(da).lower().split(";"):
            return False

    for nda in not_have:
        if answers.get(nda).lower() in not_have.get(nda).lower().split(";"):
            return False

    return True


def write_tpl(project_name, tpl, tpl_location, path):
    template = pkg_resources.read_text(tpl_location, tpl).replace(
        "application_tpl", project_name.replace("-", "_")
    )
    with open(path, "a") as file:
        file.writelines(template)


""" maybe useful
def write_templates(project_name: str, tpl_location, templates_paths: list):
    for tpl_path in templates_paths():
        tpl, path = tpl_path
        write_tpl(project_name, tpl, tpl_location, path)
"""


def pjoin(*args):
    return "/".join(list(args))


def add_requirements(pf: str, *requirements):
    with open(f"{pf}/requirements.txt", "a") as file:
        for requirement in requirements:
            file.write("{}\r\n".format(requirement))


def add_dev_requirements(pf: str, *requirements):
    with open(f"{pf}/requirements-dev.txt", "a") as file:
        for requirement in requirements:
            file.write("{}\r\n".format(requirement))


def cleanup(answers: Answers):
    ...


def make_commom(answers: Answers):
    paf = answers.application_project_folder
    pf = answers.project_folder

    os.makedirs(pjoin(pf, "tests"))
    os.makedirs(paf)
    os.makedirs(pjoin(paf, "ext"))
    os.makedirs(pjoin(paf, "models"))
    os.makedirs(pjoin(paf, "config"))
    os.makedirs(pjoin(paf, "commands"))
    os.makedirs(pjoin(paf, "examples"))
    if "admin" in answers.features:
        os.makedirs(pjoin(paf, "ext", "admin"))
    open(pjoin(paf, "__init__.py"), "a").close()
    open(pjoin(paf, "app.py"), "a").close()
    open(pjoin(paf, "ext", "__init__.py"), "a").close()
    open(pjoin(paf, "models", "__init__.py"), "a").close()
    open(pjoin(paf, "config", "__init__.py"), "a").close()
    open(pjoin(paf, "commands", "__init__.py"), "a").close()
    open(pjoin(paf, "examples", "__init__.py"), "a").close()

    add_requirements(pf, "flask")
    add_dev_requirements(pf, "black", "isort", "flake8")

    write_tpl("", ".gitignore_tpl", templates, pjoin(pf, ".gitignore"))
    write_tpl(
        answers.args.project_name, "wsgi_tpl", templates, pjoin(pf, "wsgi.py")
    )
    write_tpl(
        answers.args.project_name,
        "examples_init_tpl",
        templates.examples,
        pjoin(paf, "examples", "__init__.py"),
    )



from flask_sketch.questions.commom import commom_questions
from flask_sketch.questions.api_framework import api_framework_questions
from flask_sketch.questions.auth_framework import auth_framework_questions
from flask_sketch.questions.config import config_questions
from flask_sketch.questions.features import features_questions

questions = []

questions.extend(commom_questions)
questions.extend(api_framework_questions)
questions.extend(auth_framework_questions)
questions.extend(config_questions)
questions.extend(features_questions)


def get_questions():
    return questions



from flask_sketch.handlers.app_type_handler import app_type_handler
from flask_sketch.handlers.database_handler import database_handler
from flask_sketch.handlers.auth_handler import auth_handler
from flask_sketch.handlers.api_framework_handler import api_framework_handler
from flask_sketch.handlers.config_handler import config_handler
from flask_sketch.handlers.features_handler import handle_features

__all__ = [
    "app_type_handler",
    "database_handler",
    "auth_handler",
    "api_framework_handler",
    "config_handler",
    "handle_features",
]



from flask_sketch.utils import has_answers

api_framework_questions = [
    {
        "type": "list",
        "message": "Select your API Framework",
        "name": "api_framework",
        "choices": [
            {"name": "Flask-Restx (aka Flask-Restplus)", "value": "restx"},
            {"name": "Flask-Restless", "value": "restless"},
            {
                "name": "Flask-RESTful",
                "value": "restful",
                "disabled": "Not yet supported",
            },
            {"name": "None", "value": "none"},
        ],
        "when": lambda answers: has_answers(
            answers,
            not_have={
                "application_type": "web_only",
                "database": "mongodb;none",
            },
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
    {
        "type": "list",
        "message": "Select your API Framework",
        "name": "api_framework",
        "choices": [
            {"name": "Flask-Restx (aka Flask-Restplus)", "value": "restx"},
            {
                "name": "Flask-RESTful",
                "value": "restful",
                "disabled": "Not yet supported",
            },
            {"name": "None", "value": "none"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"database": "none"},
            not_have={"application_type": "web_only"},
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
    {
        "type": "list",
        "message": "Select your API Framework",
        "name": "api_framework",
        "choices": [
            {"name": "Flask-Restx (aka Flask-Restplus)", "value": "restx"},
            {
                "name": "Flask-RESTful",
                "value": "restful",
                "disabled": "Not yet supported",
            },
            {"name": "None", "value": "none"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"database": "mongodb"},
            not_have={"application_type": "web_only"},
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
]



import importlib.resources as pkg_resources  # noqa
from flask_sketch import templates  # noqa
from flask_sketch.utils import (
    Answers,
    GenericHandler,
    write_tpl,
    add_requirements,
    pjoin,
)


def handle_sql_db(answers: Answers):
    add_requirements(answers.project_folder, "flask-sqlalchemy")

    answers.settings["default"]["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
    answers.settings["development"]["SQLALCHEMY_TRACK_MODIFICATIONS"] = True
    answers.settings["default"][
        "SQLALCHEMY_DATABASE_URI"
    ] = "sqlite:///db.sqlite3"
    answers.settings["default"]["EXTENSIONS"].extend(
        [f"{answers.args.project_name}.ext.database:init_app"]
    )

    write_tpl(
        answers.args.project_name,
        "ext_sqlalchemy_tpl",
        templates.ext,
        pjoin(answers.application_project_folder, "ext", "database.py"),
    )


def sqlite_handler(answers: Answers):
    if answers.database == "sqlite":
        handle_sql_db(answers)
        answers.settings["production"][
            "SQLALCHEMY_DATABASE_URI"
        ] = "sqlite:///production_db.sqlite3"
        return True


def mysql_handler(answers: Answers):
    if answers.database == "mysql":
        add_requirements(answers.project_folder, "mysqlclient")
        handle_sql_db(answers)
        answers.settings["production"][
            "SQLALCHEMY_DATABASE_URI"
        ] = "mysql+mysqldb://<user>:<password>@<server_ip>/MY_DATABASE"
        return True


def postgres_handler(answers: Answers):
    if answers.database == "postgres":
        add_requirements(answers.project_folder, "psycopg2")
        handle_sql_db(answers)
        answers.settings["production"][
            "SQLALCHEMY_DATABASE_URI"
        ] = "postgres://<user>:<password>@<server_ip>/MY_DATABASE"
        return True


def mongodb_handler(answers: Answers):
    if answers.database == "mongodb":
        return True


def none_handler(answers: Answers):
    if answers.database == "none":
        return True


class DatabaseHandler(GenericHandler):
    ...


database_handler = DatabaseHandler(
    sqlite_handler,
    mysql_handler,
    postgres_handler,
    mongodb_handler,
    none_handler,
)



from flask_sketch.utils import has_answers

features_questions = [
    {
        "type": "checkbox",
        "message": "Select the frontend features.",
        "name": "frontend_features",
        "choices": [
            {"name": "Flask-HTMLmin", "disabled": "Not yet supported"},
            {"name": "Flask-Assets", "disabled": "Not yet supported"},
            {"name": "Flask-Talisman", "disabled": "Not yet supported"},
        ],
        "when": lambda answers: "TODO_IN_FUTURE"
        in answers.get("application_type"),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Migrations (Flask-Migrate)", "value": "migrate"},
            {"name": "Admin Interface (Flask-Admin)", "value": "admin"},
            {"name": "Cache (Flask-Caching)", "value": "caching"},
            {"name": "Rate Limiting (Flask-Limiter)", "value": "limiter"},
            {"name": "Flask-DebugToolbar", "value": "debugtoolbar"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"application_type": "web_only"},
            not_have={"database": "none"},
        ),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Cache (Flask-Caching)"},
            {"name": "Rate Limiting (Flask-Limiter)"},
            {"name": "Flask-DebugToolbar"},
        ],
        "when": lambda answers: has_answers(
            answers, have={"application_type": "web_only", "database": "none"},
        ),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Flask-CORS"},
            {"name": "Migrations (Flask-Migrate)"},
            {"name": "Admin Interface (Flask-Admin)"},
            {"name": "Cache (Flask-Caching)"},
            {"name": "Rate Limiting (Flask-Limiter)"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"application_type": "api_only"},
            not_have={"database": "none"},
        ),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Flask-CORS"},
            {"name": "Cache (Flask-Caching)"},
            {"name": "Rate Limiting (Flask-Limiter)"},
        ],
        "when": lambda answers: has_answers(
            answers, have={"application_type": "api_only", "database": "none"},
        ),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Flask-CORS"},
            {"name": "Migrations (Flask-Migrate)"},
            {"name": "Admin Interface (Flask-Admin)"},
            {"name": "Cache (Flask-Caching)"},
            {"name": "Rate Limiting (Flask-Limiter)"},
            {"name": "Flask-DebugToolbar"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"application_type": "web_and_api"},
            not_have={"database": "none"},
        ),
    },
    {
        "type": "checkbox",
        "message": "Select some more features for your project",
        "name": "features",
        "choices": [
            {"name": "Flask-CORS"},
            {"name": "Cache (Flask-Caching)"},
            {"name": "Rate Limiting (Flask-Limiter)"},
            {"name": "Flask-DebugToolbar"},
        ],
        "when": lambda answers: has_answers(
            answers,
            have={"application_type": "web_and_api", "database": "none"},
        ),
    },
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flask_sketch',
 'flask_sketch.handlers',
 'flask_sketch.questions',
 'flask_sketch.templates',
 'flask_sketch.templates.app',
 'flask_sketch.templates.commands',
 'flask_sketch.templates.config',
 'flask_sketch.templates.examples',
 'flask_sketch.templates.ext',
 'flask_sketch.templates.ext.admin',
 'flask_sketch.templates.models',
 'flask_sketch.templates.site']

package_data = \
{'': ['*']}

install_requires = \
['pyfiglet>=0.8.post1,<0.9', 'pyinquirer>=1.0.3,<2.0.0', 'toml>=0.10.1,<0.11.0']

entry_points = \
{'console_scripts': ['flask-sketch = flask_sketch.__main__:flask_sketch']}

setup_kwargs = {
    'name': 'flask-sketch',
    'version': '0.2.7',
    'description': 'A CLI for autogenerate folder structure and boilerplate for Flask applications',
    'long_description': '# flask-sketch\nA Python CLI for auto-generate folders structure for Flask Applications.\n',
    'author': 'Eric Souza',
    'author_email': 'ericsouza0801@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ericsouza/flask-sketch',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from flask_sketch.utils import has_answers

auth_framework_questions = [
    {
        "type": "list",
        "message": "Select the Authentication Framework",
        "name": "auth_framework",
        "choices": [
            {
                "name": "Flask-Security-Too (aka Flask-Security)",
                "value": "security_web",
            },
            {
                "name": "Flask-Login",
                "value": "login_web",
                "disabled": "Not supported yet",
            },
            {
                "name": "Flask-BasicAuth",
                "value": "basicauth_web",
                "disabled": "Not supported yet",
            },
            {"name": "None", "value": "none", "disabled": "Not supported yet"},
        ],
        "when": lambda answers: has_answers(
            answers, have={"application_type": "web_only"}
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
    {
        "type": "list",
        "message": "Select the Authentication Framework",
        "name": "auth_framework",
        "choices": [
            {"name": "Flask-Praetorian (recommended)", "value": "praetorian"},
            {
                "name": "Flask-JWT-Extended",
                "value": "jwt_extended",
                "disabled": "Not yet supported",
            },
            {"name": "Flask-BasicAuth", "value": "basicauth_api"},
            {"name": "None", "value": "none"},
        ],
        "when": lambda answers: has_answers(
            answers, have={"application_type": "api_only"}
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
    {
        "type": "list",
        "message": "Select the Authentication Framework",
        "name": "auth_framework",
        "choices": [
            {
                "name": "Flask-Security-Too + Flask-JWT-Extended",
                "value": "security_web_api",
            },
            {
                "name": "Flask-Login + Flask-JWT-Extended",
                "value": "login_jwt_extended",
            },
            {"name": "Flask-BasicAuth", "value": "basicauth_web_api"},
            {"name": "None", "value": "none"},
        ],
        "when": lambda answers: has_answers(
            answers, have={"application_type": "web_and_api"}
        ),
        "validate": lambda answer: "You must choose at least one topping."
        if len(answer) == 0
        else True,
    },
]
